#ifndef __STAN__AGRAD__MATRIX_H__
#define __STAN__AGRAD__MATRIX_H__

// global include
// #include <Eigen/Dense>
#include <Eigen/Core>
#include <Eigen/Eigenvalues>
#include <Eigen/Cholesky>
#include <Eigen/LU>
#include "stan/agrad/agrad.hpp"

/**
 * Using Eigen package namespace to define numerical traits for
 * automatic differentiation variables.  
 */
namespace Eigen {


  namespace internal {
    
    template<>  
    struct scalar_product_traits<stan::agrad::var,double> {
      typedef stan::agrad::var ReturnType;
    };

    template<>  
    struct scalar_product_traits<double,stan::agrad::var> {
      typedef stan::agrad::var ReturnType;
    };

  }

  template<> struct NumTraits<stan::agrad::var>  {
    typedef stan::agrad::var Real;
    typedef stan::agrad::var NonInteger;
    typedef stan::agrad::var Nested;
    inline static Real epsilon() { 
      return std::numeric_limits<double>::epsilon(); 
    }
    inline static Real dummy_precision() {
      return 1e-12; // copied from NumTraits.h values for double
    }
    inline static Real highest() {
      return std::numeric_limits<double>::max();
    }
    inline static Real lowest() {
      return -std::numeric_limits<double>::max();
    }

    enum {
      IsInteger = 0,
      IsSigned = 1,
      IsComplex = 0,
      RequireInitialization = 0,
      ReadCost = 1,
      AddCost = 1,
      MulCost = 1,
      HasFloatingPoint = 1,
    };
  };

}

#endif
